//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
//
//	sdrc_lite sdr sdram controller back-end : avalon wrapper
//
//	2012/05/05	version beta2.2c
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
`timescale 1ns/100ps
module MCB_AVL_WRAP(
	csi_clockreset_clk,
	csi_clockreset_reset_n,
	avs_s1_address,
	avs_s1_read,
	avs_s1_write,
	avs_s1_beginbursttransfer,
	avs_s1_waitrequest,
	avs_s1_burstcount,
	avs_s1_readdatavalid,
	avs_s1_readdata,
	avs_s1_writedata,
	mcb_bb,
	mcb_wr_n,
	mcb_bl,
	mcb_ba,
	mcb_ra,
	mcb_ca,
	mcb_busy,
	mcb_rdat_vld,
	mcb_wdat_req,
	mcb_rdat,
	mcb_wdat,
	mcb_i_ready
);
`include "../rtl/SDRC_MCB_AVL_PAR.v"
// avalon interface signals
input						csi_clockreset_clk;
input						csi_clockreset_reset_n;
input	[AVL_A_W-1:0]		avs_s1_address;
input						avs_s1_read;
input						avs_s1_write;
input						avs_s1_beginbursttransfer;
output						avs_s1_waitrequest;
input	[AVL_BC_W-1:0]		avs_s1_burstcount;
output						avs_s1_readdatavalid;
output	[AVL_D_W-1:0]		avs_s1_readdata;
input	[AVL_D_W-1:0]		avs_s1_writedata;
// memory controller back-end interface signals 
output						mcb_bb;
output						mcb_wr_n;
output	[1:0]				mcb_bl;
output	[MCB_B_W-1:0]		mcb_ba;
output	[MCB_R_W-1:0]		mcb_ra;
output	[MCB_C_W-1:0]		mcb_ca;
input						mcb_busy;
input						mcb_rdat_vld;
input						mcb_wdat_req;
input	[MCB_D_W-1:0]		mcb_rdat;
output	[MCB_D_W-1:0]		mcb_wdat;
input						mcb_i_ready;
// internal wires
wire	[1:0]				mcb_nxt_bl;
// internal registers
reg		[AVL_BC_W-1:0]		avs_s1_burstcount_m1;
reg							mcb_wdat_req_dly1;
reg							mcb_nxt_wr_n;
reg		[MCB_B_W-1:0]		mcb_nxt_ba;
reg		[MCB_R_W-1:0]		mcb_nxt_ra;
reg		[MCB_C_W-1:0]		mcb_nxt_ca;				
reg							mcb_bb;
reg							mcb_wr_n;
reg		[1:0]				mcb_bl;
reg		[MCB_B_W-1:0]		mcb_ba;
reg		[MCB_R_W-1:0]		mcb_ra;
reg		[MCB_C_W-1:0]		mcb_ca;
// avl-mm mcb wrapper logic
assign avs_s1_waitrequest		= (~mcb_i_ready) | avs_s1_beginbursttransfer |
									(avs_s1_write & (~mcb_wdat_req_dly1)) |
									(avs_s1_read & mcb_busy);
assign avs_s1_readdatavalid		= mcb_rdat_vld;
assign avs_s1_readdata				= mcb_rdat;
assign mcb_wdat					= avs_s1_writedata;
assign mcb_nxt_bl				= avs_s1_burstcount_m1[AVL_BC_W-2:AVL_BC_W-3];
always@(posedge csi_clockreset_clk, negedge csi_clockreset_reset_n)
	if(csi_clockreset_reset_n == 1'b0)
		mcb_nxt_wr_n			<= 1'b0;
	else if((avs_s1_beginbursttransfer == 1'b1) & (avs_s1_read == 1'b1))
		mcb_nxt_wr_n			<= 1'b1;
	else if((avs_s1_beginbursttransfer == 1'b1) & (avs_s1_write == 1'b1))
		mcb_nxt_wr_n			<= 1'b0;
	else
		mcb_nxt_wr_n			<= mcb_nxt_wr_n;
always@(posedge csi_clockreset_clk, negedge csi_clockreset_reset_n)
	if(csi_clockreset_reset_n == 1'b0) begin
		avs_s1_burstcount_m1	<= 0;
		mcb_nxt_ba				<= 0;
		mcb_nxt_ra				<= 0;
		mcb_nxt_ca				<= 0;
	end
	else if(avs_s1_beginbursttransfer == 1'b1) begin
		avs_s1_burstcount_m1	<= avs_s1_burstcount
									- {{(AVL_BC_W-1){1'b0}}, 1'b1};
		mcb_nxt_ba				<= avl_addr_to_mcb_ba(avs_s1_address);
		mcb_nxt_ra				<= avl_addr_to_mcb_ra(avs_s1_address);
		mcb_nxt_ca				<= avl_addr_to_mcb_ca(avs_s1_address);
	end
	else begin
		avs_s1_burstcount_m1	<= avs_s1_burstcount_m1;
		mcb_nxt_ba				<= mcb_nxt_ba;
		mcb_nxt_ra				<= mcb_nxt_ra;
		mcb_nxt_ca				<= mcb_nxt_ca;
	end
always@(posedge csi_clockreset_clk, negedge csi_clockreset_reset_n)
	if(csi_clockreset_reset_n == 1'b0)
		mcb_wdat_req_dly1		<= 1'b0;
	else
		mcb_wdat_req_dly1		<= mcb_wdat_req;
always@(posedge csi_clockreset_clk, negedge csi_clockreset_reset_n)
	if(csi_clockreset_reset_n == 1'b0) begin
		mcb_bb					<= 1'b0;
		mcb_wr_n				<= 1'b0;
		mcb_bl					<= 0;
		mcb_ba					<= 0;
		mcb_ra					<= 0;
		mcb_ca					<= 0;
	end
	else if(((avs_s1_read == 1'b1) | (avs_s1_write == 1'b1)) & (mcb_busy == 1'b0) 
		& (avs_s1_beginbursttransfer == 1'b0)) begin
		mcb_bb					<= 1'b1;
		mcb_wr_n				<= mcb_nxt_wr_n;
		mcb_bl					<= mcb_nxt_bl;
		mcb_ba					<= mcb_nxt_ba;
		mcb_ra					<= mcb_nxt_ra;
		mcb_ca					<= mcb_nxt_ca;
	end
	else begin
		mcb_bb					<= 1'b0;
		mcb_wr_n				<= mcb_wr_n;
		mcb_bl					<= mcb_bl;
		mcb_ba					<= mcb_ba;
		mcb_ra					<= mcb_ra;
		mcb_ca					<= mcb_ca;
	end
endmodule
